#include <csignal>
#include "cpu.hpp"
#include "gui.hpp"

#include <pthread.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/input.h>

namespace GUI {

#define MAX_KEYCODE	64
#define LOCAL_SIZE	(480 * 512 * 2)
#define FBMAP_SIZE	(480 * 800 * 2)

char key_table[MAX_KEYCODE] = {0};
unsigned short *framebuffer = NULL;
unsigned short *local = NULL;

void* keyboard_thread(void *arg)
{
	struct input_event ie;
	int kf;

	kf = open("/dev/input/event0", O_RDONLY);
	if (kf < 0) {
		printf("keyboard is not exist, input ignore.\n");
		return NULL;
	}

	while (1) {
		int size;
		size = read(kf, &ie, sizeof(struct input_event));
		if (size != sizeof(struct input_event))
			break;

		if (ie.type != EV_KEY || ie.code >= MAX_KEYCODE)
			continue;

		key_table[ie.code] = ie.value;
		if (ie.code == KEY_ESC)
			break;
	};

	close(kf);
	return arg;
}

/* Initialize GUI */
void init()
{
    // create thread to listen to keyboard event.
    pthread_t tid;
    pthread_create(&tid, NULL, keyboard_thread, NULL);

    // map framebuffer to local, so we can access it directly.
    int fd = open("/dev/fb0", O_RDWR);
    if (fd <= 0) {
	    printf("frame buffer is not exist, exit.\n");
	    exit(-1);
    }

    framebuffer = (unsigned short *)mmap(NULL, FBMAP_SIZE, PROT_WRITE, MAP_SHARED, fd, 0);
    if (framebuffer == NULL) {
	    printf("can not map to frame buffer, exit.\n");
	    exit(-1);
    }
}

/* Send the rendered frame to the GUI */
void new_frame(u16* pixels)
{
    memcpy(framebuffer, pixels, LOCAL_SIZE);
}

/* Get the joypad state from SDL */
u8 get_joypad_state(int n)
{
    u8 j = 0;
    if (n == 0) {
	j |= key_table[KEY_K] << 0;
	j |= key_table[KEY_J] << 1;
	j |= key_table[KEY_U] << 2;
	j |= key_table[KEY_I] << 3;
	j |= key_table[KEY_W] << 4;
	j |= key_table[KEY_S] << 5;
	j |= key_table[KEY_A] << 6;
	j |= key_table[KEY_D] << 7;
    }
    return j;
}

/* Run the emulator */
void run()
{
    bool pause = false;
    while (true) {
	if (key_table[KEY_ESC])
		break;
	if (not pause)
		CPU::run_frame();
	usleep(2200);
    }
}


}
